/**
 * Copyright (C) 2003-2004 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package sync4j.examples.admin;

import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.Serializable;

import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JTextField;
import javax.swing.SwingConstants;
import javax.swing.border.TitledBorder;

import sync4j.framework.engine.source.ContentType;
import sync4j.framework.engine.source.SyncSource;
import sync4j.framework.engine.source.SyncSourceInfo;

import sync4j.syncadmin.ui.ManagementPanel;

import sync4j.examples.engine.source.DummySyncSource;

import org.apache.commons.lang.StringUtils;

/**
 * This class implements the configuration panel for an DummySyncSource
 *
 * @author Fabio Maggi
 *
 * @version $Id: DummySyncSourceConfigPanel.java,v 1.2 2004/06/06 20:42:33 stefano_fornari Exp $
 */
public class DummySyncSourceConfigPanel
extends ManagementPanel
implements Serializable {
    
    // --------------------------------------------------------------- Constants
    
    /**
     * Allowed characters for name and uri
     */
    public static final String NAME_ALLOWED_CHARS
    = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890-_.";
    
    // ------------------------------------------------------------ Private data
    /** label for the panel's name */
    private JLabel panelName = new JLabel();
    
    /** border to evidence the title of the panel */
    private TitledBorder  titledBorder1;
    
    private JLabel           nameLabel          = new JLabel()     ;
    private JTextField       nameValue          = new JTextField() ;
    private JLabel           typeLabel          = new JLabel()     ;
    private JTextField       typeValue          = new JTextField() ;
    private JLabel           sourceUriLabel     = new JLabel()     ;
    private JTextField       sourceUriValue     = new JTextField() ;
    
    private JButton          confirmButton      = new JButton()    ;
    
    private DummySyncSource  syncSource         = null             ;
    
    // ------------------------------------------------------------ Constructors
    
    /**
     * Creates a new DummySyncSourceConfigPanel instance
     */
    public DummySyncSourceConfigPanel() {
        init();
    }
    
    // ----------------------------------------------------------- Private methods
    
    /**
     * Create the panel
     * @throws Exception if error occures during creation of the panel
     */
    private void init(){
        // set layout
        this.setLayout(null);
        
        // set properties of label, position and border
        //  referred to the title of the panel
        titledBorder1 = new TitledBorder("");
        
        panelName.setFont(titlePanelFont);
        panelName.setText("Edit Dummy SyncSource");
        panelName.setBounds(new Rectangle(14, 5, 316, 28));
        panelName.setAlignmentX(SwingConstants.CENTER);
        panelName.setBorder(titledBorder1);
        
        sourceUriLabel.setText("Source URI: ");
        sourceUriLabel.setFont(defaultFont);
        sourceUriLabel.setBounds(new Rectangle(14, 60, 150, 18));
        sourceUriValue.setFont(new java.awt.Font("Arial", 0, 12));
        sourceUriValue.setBounds(new Rectangle(170, 60, 350, 18));
        
        nameLabel.setText("Name: ");
        nameLabel.setFont(defaultFont);
        nameLabel.setBounds(new Rectangle(14, 90, 150, 18));
        nameValue.setFont(new java.awt.Font("Arial", 0, 12));
        nameValue.setBounds(new Rectangle(170, 90, 350, 18));
        
        typeLabel.setText("Type: ");
        typeLabel.setFont(defaultFont);
        typeLabel.setBounds(new Rectangle(14, 120, 150, 18));
        typeValue.setFont(new java.awt.Font("Arial", 0, 12));
        typeValue.setBounds(new Rectangle(170, 120, 350, 18));
        
        confirmButton.setFont(defaultFont);
        confirmButton.setText("Add");
        confirmButton.setBounds(170, 200, 70, 25);
        
        confirmButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent event ) {
                try {
                    validateValues();
                    getValues();
                    if (setSyncSource(syncSource)) {
                        sourceUriValue.setEditable(false);
                        confirmButton.setText("Save");
                    }
                } catch (Exception e) {
                    error(e.getMessage());
                }
            }
        });
        
        // add all components to the panel
        this.add(panelName        , null);
        this.add(nameLabel        , null);
        this.add(nameValue        , null);
        this.add(typeLabel        , null);
        this.add(typeValue        , null);
        this.add(sourceUriLabel   , null);
        this.add(sourceUriValue   , null);
        this.add(confirmButton    , null);
        
    }
    
    /**
     * Loads the given syncSource showing the name, uri and type in the pael's
     * fields.
     *
     * @param syncSource the SyncSource instance
     */
    public void loadSyncSource(SyncSource syncSource) {
        if (!(syncSource instanceof DummySyncSource)) {
            error("This is not a DummySyncSource! Unable to process SyncSource values.");
            return;
        }
        if (getState() == ManagementPanel.STATE_INSERT) {
            confirmButton.setText("Add");
        } else if (getState() == ManagementPanel.STATE_UPDATE) {
            confirmButton.setText("Save");
        }
        
        this.syncSource = (DummySyncSource)syncSource;
        
        sourceUriValue.setText(this.syncSource.getSourceURI() );
        nameValue.setText     (this.syncSource.getName()      );
        
        if (this.syncSource.getSourceURI() != null) {
            sourceUriValue.setEditable(false);
        }
    }
    
    // ----------------------------------------------------------- Private methods
    /**
     * Checks if the values provided by the user are all valid. In caso of errors,
     * a IllegalArgumentException is thrown.
     *
     * @throws IllegalArgumentException if:
     *         <ul>
     *         <li>name, uri, type or directory are empty (null or zero-length)
     *         <li>the types list length does not match the versions list length
     *         </ul>
     */
    private void validateValues() throws IllegalArgumentException {
        String value = null;
        
        value = nameValue.getText();
        if (StringUtils.isEmpty(value)) {
            throw new
            IllegalArgumentException(
            "Field 'Name' cannot be empty. Please provide a SyncSource name.");
        }
        
        if (!StringUtils.containsOnly(value, NAME_ALLOWED_CHARS.toCharArray())) {
            throw new
            IllegalArgumentException(
            "Only the following characters are allowed for field 'Name': \n" + NAME_ALLOWED_CHARS);
        }
        
        value = typeValue.getText();
        if (StringUtils.isEmpty(value)) {
            throw new
            IllegalArgumentException(
            "Field 'Type' cannot be empty. Please provide a SyncSource type.");
        }
        
        value = sourceUriValue.getText();
        if (StringUtils.isEmpty(value)) {
            throw new
            IllegalArgumentException(
            "Field 'Source URI' cannot be empty. Please provide a SyncSource URI.");
        }
    }
    
    /**
     * Set syncSource properties with the values provided by the user.
     */
    private void getValues() {
        syncSource.setSourceURI      (sourceUriValue.getText().trim());
        syncSource.setName           (nameValue.getText().trim()     );
        syncSource.setType           (typeValue.getText().trim()     );
        
        ContentType[] contentTypes = new ContentType[] {
            new ContentType("text/plain", "1.0")
        };
        
        
        syncSource.setInfo(new SyncSourceInfo(contentTypes, 0));
    }
    
    private void error(String msg) {
        JOptionPane.showMessageDialog(null, msg, "Error", JOptionPane.ERROR_MESSAGE);
    }
}